/*
  Copyright (c) 2005 by Jakob Schroeter <js@camaya.net>
  This file is part of the joox project. http://camaya.net/joox

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warrenty.
*/


#include "joox.h"
#include "contactlist.h"
#include "xmlconsole.h"

#include <gloox/client.h>
#include <gloox/jid.h>
#include <gloox/rostermanager.h>

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Menu_Bar.H>

using namespace gloox;

Joox::Joox( int x, int y )
  : Fl_Window( x, y ), m_xml( false )
{
  Fl_Menu_Item menuitems[] = {
    { "&Connection",    0, 0, 0, FL_SUBMENU },
    { "C&onnect",       FL_CTRL + 'o', (Fl_Callback *)connect_cb, this },
    { "&Disconnect",    FL_CTRL + 'd', (Fl_Callback *)disconnect_cb, this },
    { "&XML Console",   FL_CTRL + 'x', (Fl_Callback *)xmlconsole_cb, this },
    { "&Quit",          FL_CTRL + 'q', (Fl_Callback *)quit_cb, this },
    { 0 },
  { 0 }
  };
  Fl_Menu_Bar *m = new Fl_Menu_Bar( 0, 0, 640, 30 );
  m->copy( menuitems );

  ContactList* cl = new ContactList( 0, 0, 640, 30, "cl" );

  JID id( "hurkhurk@jabber.cc/joox" );
  m_client = new Client( id, "hurkhurks" );
  m_client->setAutoPresence( true );
  m_client->setInitialPriority( 5 );
  m_client->registerConnectionListener( this );
  m_client->rosterManager()->registerRosterListener( cl );
}

Joox::~Joox()
{
  delete m_client;
}

void Joox::connect()
{
  if( m_client->connect( false ) )
    Fl::add_timeout( 0.1, timeout_cb, m_client );
}

void Joox::disconnect()
{
  m_client->disconnect();
}

void Joox::quit()
{
  m_client->disconnect();
  exit( 0 );
}

void Joox::xmlconsole()
{
  if( !m_xml )
  {
    m_xml = true;
    XMLConsole *xml = new XMLConsole( m_client, 600, 500 );
    xml->show();
  }
}

void Joox::onConnect()
{
  printf( "connected\n" );
}

void Joox::onDisconnect( ConnectionError e )
{
  printf( "disconnected\n" );
}

void connect_cb( Fl_Widget*, void *v )
{
  ((Joox*)v)->connect();
}

void disconnect_cb( Fl_Widget*, void *v )
{
  ((Joox*)v)->disconnect();
}

void quit_cb( Fl_Widget*, void *v )
{
  ((Joox*)v)->quit();
}

void timeout_cb( void *v )
{
  Client* c = (Client*)v;
  if( c->recv( 0 ) == CONN_OK )
    Fl::add_timeout( 0.1, timeout_cb, v );
}

void xmlconsole_cb( Fl_Widget*, void *v )
{
  ((Joox*)v)->xmlconsole();
}
